@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2023 Derick Rethans                               |
+   | Copyright (c) 2002-2024 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.01 of the Xdebug license,   |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -22,6 +22,7 @@
 #include "TSRM.h"
 #include "php_globals.h"
 #include "Zend/zend_alloc.h"
+#include "Zend/zend_exceptions.h"
 
 #include "php_xdebug.h"
 #include "profiler.h"
@@ -44,8 +45,10 @@ void xdebug_init_profiler_globals(xdebug_profiler_globals_t *xg)
 
 void xdebug_profiler_minit(void)
 {
+#if PHP_VERSION_ID < 80400
 	/* Overload the "exit" opcode */
 	xdebug_set_opcode_handler(ZEND_EXIT, xdebug_profiler_exit_handler);
+#endif
 }
 
 void xdebug_profiler_mshutdown(void)
@@ -82,6 +85,15 @@ void xdebug_profiler_pcntl_exec_handler(void)
 	deinit_if_active();
 }
 
+void xdebug_profiler_exit_function_handler(void)
+{
+	function_stack_entry *fse = XDEBUG_VECTOR_TAIL(XG_BASE(stack));
+
+	deinit_if_active();
+
+	xdebug_profiler_free_function_details(fse);
+}
+
 int xdebug_profiler_exit_handler(XDEBUG_OPCODE_HANDLER_ARGS)
 {
 	const zend_op *cur_opcode = execute_data->opline;
@@ -516,7 +528,7 @@ void xdebug_profiler_function_end(function_stack_entry *fse)
 	xdebug_str_addc(&file_buffer, '\n');
 
 	xdebug_file_write(file_buffer.d, sizeof(char), file_buffer.l, &XG_PROF(profile_file));
-	xdebug_str_dtor(file_buffer);
+	xdebug_str_destroy(&file_buffer);
 }
 
 void xdebug_profiler_free_function_details(function_stack_entry *fse)
